@@ -1,9 +1,10 @@
 import greenlet, logging, os, os.path
 from collections import deque
-from mixins import mixins
-from util import VimError, VimExit
+from .mixins import mixins
+from .util import VimError, VimExit, decode_obj
 from traceback import format_exc
-import cProfile, pstats, StringIO
+import cProfile, pstats, sys
+from io import StringIO
 
 logger = logging.getLogger(__name__)
 debug, info, warn = (logger.debug, logger.info, logger.warn,)
@@ -44,24 +45,26 @@ class Client(object):
         self.vim = None
         self.loop_running = False
         self.pending = deque()
+        self._discover_api()
+        if sys.version_info[0] > 2:
+            self.stream.encoding = self.vim.get_option('encoding').decode('ascii')
+        else:
+            self.stream.encoding = self.vim.get_option('encoding')
 
-
-    def rpc_yielding_request(self, method, args):
+    def rpc_yielding_request(self, method, args, decode_str=None):
         gr = greenlet.getcurrent()
         parent = gr.parent
 
         def response_cb(err, result):
-            pass # replaces next logging statement
-            #debug('response is available for greenlet %s, switching back', gr)
+            debug('response is available for greenlet %s, switching back', gr)
             gr.switch(err, result)
 
-        self.stream.send(method, args, response_cb)
-        pass # replaces next logging statement
-        #debug('yielding from greenlet %s to wait for response', gr)
+        self.stream.send(method, args, response_cb, decode_str=decode_str)
+        debug('yielding from greenlet %s to wait for response', gr)
         return parent.switch()
 
 
-    def rpc_blocking_request(self, method, args):
+    def rpc_blocking_request(self, method, args, decode_str=None):
         response = {}
 
         def response_cb(err, result):
@@ -69,16 +72,14 @@ class Client(object):
             response['result'] = result
             self.stream.loop_stop()
 
-        pass # replaces next logging statement
-        #debug('will now perform a blocking rpc request: %s, %s', method, args)
-        self.stream.send(method, args, response_cb)
+        debug('will now perform a blocking rpc request: %s, %s', method, args)
+        self.stream.send(method, args, response_cb, decode_str=decode_str)
         queue = []
 
         while not response:
             msg = self.next_message()
             if msg:
-                pass # replaces next logging statement
-                #debug('message received while waiting for response: %s', msg)
+                debug('message received while waiting for response: %s', msg)
                 queue.append(msg)
 
         self.pending.extend(queue)
@@ -86,14 +87,14 @@ class Client(object):
         return response.get('err', None), response.get('result', None)
 
 
-    def rpc_request(self, method, args, expected_type=None):
+    def rpc_request(self, method, args, expected_type=None, decode_str=None):
         """
         Sends a rpc request to Neovim.
         """
         if self.loop_running:
-            err, result = self.rpc_yielding_request(method, args)
+            err, result = self.rpc_yielding_request(method, args, decode_str=decode_str)
         else:
-            err, result = self.rpc_blocking_request(method, args)
+            err, result = self.rpc_blocking_request(method, args, decode_str=decode_str)
 
         if err:
             raise VimError(err)
@@ -108,8 +109,7 @@ class Client(object):
                 # never leave this state
                 self.pending.appendleft(msg)
                 raise msg[1]
-            pass # replaces next logging statement
-            #debug('returning message: %s', msg)
+            debug('returning message: %s', msg)
             return msg
 
 
@@ -122,6 +122,8 @@ class Client(object):
         if msg:
             return msg
 
+        error = {}
+
         def request_cb(name, args, reply_fn):
             self.pending.append(('request', name, args, reply_fn,))
             self.stream.loop_stop()
@@ -131,13 +133,17 @@ class Client(object):
             self.stream.loop_stop()
 
         def error_cb(err):
-            self.pending.append(('eof', err,))
+            error['err'] = err
+            if isinstance(err, VimExit):
+                self.pending.append(('eof', err,))
             self.stream.loop_stop()
 
-        pass # replaces next logging statement
-        #debug('will block until a message is available')
+        debug('will block until a message is available')
         self.stream.loop_start(request_cb, notification_cb, error_cb)
 
+        if error:
+            raise error['err']
+
         msg = self.next_pending_message()
         if msg:
             return msg
@@ -151,27 +157,22 @@ class Client(object):
         def request_handler():
             try:
                 rv = self.request_cb(name, args)
-                pass # replaces next logging statement
-                #debug('greenlet %s completed, sending %s as response', gr, rv)
+                debug('greenlet %s completed, sending %s as response', gr, rv)
                 reply_fn(rv)
             except Exception as e:
                 if self.loop_running:
                     err_str = format_exc(5)
-                    pass # replaces next logging statement
-                    #warn("error caught while processing call '%s %s': %s",
-                         #name,
-                         #args,
-                         #err_str)
+                    warn("error caught while processing call '%s %s': %s",
+                         name,
+                         args,
+                         err_str)
                     reply_fn(err_str, error=True)
-                    pass # replaces next logging statement
-                    #debug('sent "%s" as response', err_str)
-            pass # replaces next logging statement
-            #debug('greenlet %s is now dying...', gr)
+                    debug('sent "%s" as response', err_str)
+            debug('greenlet %s is now dying...', gr)
             self.greenlets.remove(gr)
 
         gr = greenlet.greenlet(request_handler)
-        pass # replaces next logging statement
-        #debug('received rpc request, greenlet %s will handle it', gr)
+        debug('received rpc request, greenlet %s will handle it', gr)
         self.greenlets.add(gr)
         gr.switch()
 
@@ -180,30 +181,25 @@ class Client(object):
         def notification_handler():
             try:
                 self.notification_cb(name, args)
-                pass # replaces next logging statement
-                #debug('greenlet %s completed', gr)
+                debug('greenlet %s completed', gr)
             except Exception as e:
                 if self.loop_running:
                     err_str = format_exc(5)
-                    pass # replaces next logging statement
-                    #warn("error caught while processing event '%s %s': %s",
-                         #name,
-                         #args,
-                         #err_str)
-            pass # replaces next logging statement
-            #debug('greenlet %s is now dying...', gr)
+                    warn("error caught while processing event '%s %s': %s",
+                         name,
+                         args,
+                         err_str)
+            debug('greenlet %s is now dying...', gr)
             self.greenlets.remove(gr)
 
         gr = greenlet.greenlet(notification_handler)
-        pass # replaces next logging statement
-        #debug('received rpc notification, greenlet %s will handle it', gr)
+        debug('received rpc notification, greenlet %s will handle it', gr)
         self.greenlets.add(gr)
         gr.switch()
 
 
     def on_error(self, err):
-        pass # replaces next logging statement
-        #warn('caught error: %s', err)
+        warn('caught error: %s', err)
         self.error_cb(err)
 
 
@@ -212,16 +208,14 @@ class Client(object):
 
         try:
             assert not self.loop_running
-            pass # replaces next logging statement
-            #info('starting message loop')
+            info('starting message loop')
             self.request_cb = request_cb
             self.notification_cb = notification_cb
             self.error_cb = error_cb
             self.loop_running = True
 
             if profiling:
-                pass # replaces next logging statement
-                #info('starting profiler')
+                info('starting profiler')
                 pr = cProfile.Profile()
                 pr.enable()
 
@@ -240,17 +234,15 @@ class Client(object):
             if profiling:
                 pr.disable()
                 report = os.path.abspath('.nvim-python-client.profile')
-                pass # replaces next logging statement
-                #info('stopped profiler, writing report to %s', report)
-                s = StringIO.StringIO()
+                info('stopped profiler, writing report to %s', report)
+                s = StringIO()
                 ps = pstats.Stats(pr, stream=s)
                 ps.strip_dirs().sort_stats('tottime').print_stats(30)
 
                 with open(report, 'w') as f:
                     f.write(s.getvalue())
 
-            pass # replaces next logging statement
-            #info('exiting message loop')
+            info('exiting message loop')
             self.loop_running = False
             self.request_cb = None
             self.notification_cb = None
@@ -262,7 +254,7 @@ class Client(object):
         self.stream.loop_stop()
 
 
-    def discover_api(self):
+    def _discover_api(self):
         """
         Discovers the remote API using the special method '0'. After this
         the client will have a `vim` attribute containing an object
@@ -272,7 +264,12 @@ class Client(object):
         if self.vim:
             # Only need to do this once
             return
+        # When calling this method, &encoding was not defined yet so we will get
+        # binary strings for all content. For Python3 decode api binary strings
+        # as utf8
         channel_id, api = self.rpc_request("vim_get_api_info", [])
+        if sys.version_info[0] > 2:
+            api = decode_obj(api, 'utf8')
         # The 'Vim' class is the main entry point of the api
         types = {'vim': type('Vim', (), {})}
         setattr(types['vim'], 'loop_start',
@@ -337,11 +334,12 @@ def generate_wrapper(client, klass, name, fid, return_type, parameters):
             # functions of the vim object don't need 'self'
             args = args[1:]
         argv = []
+        decode_str = kwargs.get('decode_str', None)
         # fill with positional arguments
         for i, arg in enumerate(args):
             # Add to the argument vector 
             argv.append(arg)
-        return client.rpc_request(fid, argv, return_type)
+        return client.rpc_request(fid, argv, return_type, decode_str=decode_str)
 
     setattr(klass, name, func)
 
