@@ -1,19 +1,29 @@
 from collections import deque
+from .util import decode_obj
 import logging
 
 logger = logging.getLogger(__name__)
 debug, info, warn = (logger.debug, logger.info, logger.warn,)
 
 class RPCStream(object):
-    def __init__(self, stream):
+    def __init__(self, stream, decode_str=None):
         self.stream = stream
         self.pending_requests = {}
         self.next_request_id = 1
         self.interrupted = False
         self.stopped = False
         self.running = False
+        self._decode_str=decode_str
+        self.encoding = None
         self.posted_notifications = deque()
 
+    def decode_obj(self, obj):
+        """
+        Decode binary strings inside the object and return new object
+        """
+        if self.encoding:
+            return decode_obj(obj, self.encoding)
+        return obj
 
     def configure(self, vim):
         self.stream.configure(vim)
@@ -24,14 +34,15 @@ class RPCStream(object):
         self.stream.interrupt()
 
 
-    def send(self, method, args, response_cb):
+    def send(self, method, args, response_cb, decode_str=None):
         request_id = self.next_request_id
         # Update request id
         self.next_request_id = request_id + 1
         # Send the request
         self.stream.send([0, request_id, method, args])
-        # set the callback
-        self.pending_requests[request_id] = response_cb
+        # set the callback/ str encoding
+        decode_str = decode_str if decode_str != None else self._decode_str
+        self.pending_requests[request_id] = response_cb,decode_str
 
 
     def loop_start(self, request_cb, notification_cb, error_cb):
@@ -42,33 +53,39 @@ class RPCStream(object):
                 #   - msg[1]: id
                 #   - msg[2]: method name
                 #   - msg[3]: arguments
-                pass # replaces next logging statement
-                #debug('received request: %s, %s', msg[2], msg[3])
-                request_cb(msg[2], msg[3], reply_fn(self.stream, msg[1]))
+                debug('received request: %s, %s', msg[2], msg[3])
+                if self._decode_str:
+                    request_cb(self.decode_obj(msg[2]), self.decode_obj(msg[3]), reply_fn(self.stream, msg[1]))
+                else:
+                    request_cb(msg[2], msg[3], reply_fn(self.stream, msg[1]))
             elif msg_type == 1:
                 # response to a previous request:
                 #   - msg[1]: the id
                 #   - msg[2]: error(if any)
                 #   - msg[3]: result(if not errored)
-                pass # replaces next logging statement
-                #debug('received response: %s, %s', msg[2], msg[3])
-                self.pending_requests.pop(msg[1])(msg[2], msg[3])
+                debug('received response: %s, %s', msg[2], msg[3])
+
+                fn,decode_str = self.pending_requests.pop(msg[1])
+                if decode_str:
+                    fn(self.decode_obj(msg[2]), self.decode_obj(msg[3]))
+                else:
+                    fn(msg[2], msg[3])
             elif msg_type == 2:
                 # notification/event
                 #   - msg[1]: event name
                 #   - msg[2]: arguments
-                pass # replaces next logging statement
-                #debug('received notification: %s, %s', msg[1], msg[2])
-                notification_cb(msg[1], msg[2])
+                debug('received notification: %s, %s', msg[1], msg[2])
+                if self._decode_str:
+                    notification_cb(self.decode_obj(msg[1]), self.decode_obj(msg[2]))
+                else:
+                    notification_cb(msg[1], msg[2])
             else:
                 error = 'Received invalid message %s' % msg
-                pass # replaces next logging statement
-                #warn(error)
+                warn(error)
                 raise Exception(error)
 
         self._run(msg_cb, notification_cb, error_cb)
-        pass # replaces next logging statement
-        #debug('exiting rpc stream loop')
+        debug('exiting rpc stream loop')
 
 
 
