@@ -1,7 +1,6 @@
 from collections import deque
-from util import VimExit
-from signal import SIGTERM
-import sys, pyuv, logging
+from .util import VimExit
+import sys, pyuv, logging, signal
 
 logger = logging.getLogger(__name__)
 debug, warn = (logger.debug, logger.warn,)
@@ -11,10 +10,9 @@ class UvStream(object):
     Stream abstraction implemented on top of libuv
     """
     def __init__(self, address=None, port=None, spawn_argv=None):
-        pass # replaces next logging statement
-        #debug('initializing UvStream instance')
+        debug('initializing UvStream instance')
         self._loop = pyuv.Loop()
-        self._error = None
+        self._fatal = None
         self._connected = False
         self._data_cb = None
         self._error_cb = None
@@ -22,7 +20,11 @@ class UvStream(object):
         self._pending_writes = 0
         self._async = pyuv.Async(self._loop, self._on_async)
         self._term = pyuv.Signal(self._loop)
-        self._term.start(self._on_term, SIGTERM)
+        self._term.start(self._on_signal, signal.SIGTERM)
+        self._int = pyuv.Signal(self._loop)
+        self._int.start(self._on_signal, signal.SIGINT)
+        self._signames = dict((k, v) for v, k in signal.__dict__.items() \
+                              if v.startswith('SIG'))
         self._error_stream = None
         # Select the type of handle
         if spawn_argv:
@@ -37,24 +39,21 @@ class UvStream(object):
 
     def init_tcp(self, address, port):
         # tcp/ip
-        pass # replaces next logging statement
-        #debug('TCP address was provided, connecting...')
+        debug('TCP address was provided, connecting...')
         self._stream = pyuv.TCP(self._loop)
         self._stream.connect((address, port), self._on_connect)
 
 
     def init_pipe(self, address):
         # named pipe or unix socket
-        pass # replaces next logging statement
-        #debug('Pipe address was provided, connecting...')
+        debug('Pipe address was provided, connecting...')
         self._stream = pyuv.Pipe(self._loop)
         self._stream.connect(address, self._on_connect)
 
 
     def init_stdio(self):
         # stdin/stdout
-        pass # replaces next logging statement
-        #debug('No addresses were provided, will use stdin/stdout')
+        debug('No addresses were provided, will use stdin/stdout')
         self._read_stream = pyuv.Pipe(self._loop) 
         self._read_stream.open(sys.stdin.fileno())
         self._write_stream = pyuv.Pipe(self._loop) 
@@ -88,21 +87,18 @@ class UvStream(object):
         self.loop_stop()
         if error:
             msg = pyuv.errno.strerror(error)
-            self._error = msg
-            pass # replaces next logging statement
-            #warn('error connecting to neovim: %s', msg)
+            self._fatal = msg
+            warn('error connecting to neovim: %s', msg)
             self._connection_error = IOError(msg)
             return
         self._connected = True
         self._read_stream = self._write_stream = stream
 
 
-    def _on_term(self, handle, signum):
+    def _on_signal(self, handle, signum):
         self.loop_stop()
-        self._error = 'Received SIGTERM'
-        err = IOError(self._error)
+        err = Exception('Received %s' % self._signames[signum])
         if not self._error_cb:
-            self._loop.stop()
             raise err
         self._error_cb(err)
 
@@ -121,24 +117,22 @@ class UvStream(object):
 
     def _on_exit(self, handle, exit_status, term_signal):
         self._loop.stop()
-        self._error = (
+        self._fatal = (
             'The child nvim instance exited with status %s' % exit_status)
 
 
     def _on_stderr_read(self, handle, data, error):
         if error or not data:
             msg = pyuv.errno.strerror(error)
-            pass # replaces next logging statement
-            #warn('Error reading child nvim stderr: %s', msg)
+            warn('Error reading child nvim stderr: %s', msg)
             err = IOError(msg)
             if not self._error_cb:
                 self._loop.stop()
-                self._error = err
+                self._fatal = err
                 raise err
             self._error_cb(err)
         else:
-            pass # replaces next logging statement
-            #warn('nvim stderr: %s', data)
+            warn('nvim stderr: %s', data)
 
 
     def _on_read(self, handle, data, error):
@@ -147,20 +141,17 @@ class UvStream(object):
         """
         if error:
             msg = pyuv.errno.strerror(error)
-            pass # replaces next logging statement
-            #warn('error reading data: %s', msg)
+            warn('error reading data: %s', msg)
             self._error_cb(IOError(msg))
             self._loop.stop()
-            self._error = msg
+            self._fatal = msg
         elif not data:
-            pass # replaces next logging statement
-            #warn('connection was closed by neovim')
+            warn('connection was closed by neovim')
             self._loop.stop()
-            self._error = 'EOF'
-            self._error_cb(IOError(self._error))
+            self._fatal = 'EOF'
+            self._error_cb(IOError(self._fatal))
         else:
-            pass # replaces next logging statement
-            #debug('successfully read %d bytes of data', len(data))
+            debug('successfully read %d bytes of data', len(data))
             self._data_cb(data)
 
 
@@ -186,23 +177,20 @@ class UvStream(object):
             if error:
                 self._loop.stop()
                 msg = pyuv.errno.strerror(error)
-                self._error = msg
-                pass # replaces next logging statement
-                #warn('error writing data: %s', msg)
+                self._fatal = msg
+                warn('error writing data: %s', msg)
                 self._error_cb(IOError(msg))
-            pass # replaces next logging statement
-            #debug('successfully wrote %d bytes of data', data_len)
+            debug('successfully wrote %d bytes of data', data_len)
 
-        pass # replaces next logging statement
-        #debug('writing %d bytes of data', data_len)
+        debug('writing %d bytes of data', data_len)
         # queue the data for writing
         self._write_stream.write(data, write_cb)
 
 
     def loop_start(self, data_cb, error_cb):
-        if self._error:
-            raise IOError('An error was raised and the connection ' +
-                          'was permanently closed: %s', self._error)
+        if self._fatal:
+            raise IOError('A fatal error was raised and the connection ' +
+                          'was permanently closed: %s', self._fatal)
         if not self._connected:
             self._connect()
             if self._connection_error:
@@ -215,11 +203,9 @@ class UvStream(object):
         self._read_stream.start_read(self._on_read)
         if self._error_stream:
             self._error_stream.start_read(self._on_stderr_read)
-        pass # replaces next logging statement
-        #debug('entering libuv event loop')
+        debug('entering libuv event loop')
         self._loop.run(pyuv.UV_RUN_DEFAULT)
-        pass # replaces next logging statement
-        #debug('exited libuv event loop')
+        debug('exited libuv event loop')
         if self._error_stream:
             self._error_stream.stop_read()
         self._read_stream.stop_read()
@@ -232,5 +218,4 @@ class UvStream(object):
         Stops the event loop
         """
         self._loop.stop()
-        pass # replaces next logging statement
-        #debug('stopped event loop')
+        debug('stopped event loop')
