@@ -1,5 +1,5 @@
 import os
-from util import RemoteSequence, RemoteMap, Current, VimError
+from .util import RemoteSequence, RemoteMap, Current, VimError
 
 os_chdir = os.chdir
 os_fchdir = os.fchdir
